from .__set_data__ import Paths

__root__: str = getattr(Paths, "root")
__data__: str = getattr(Paths, "data")
__gui__: str = getattr(Paths, "gui")
__scraper__: str = getattr(Paths, "scraper")
__utils__: str = getattr(Paths, "utils")
__icons__: str = getattr(Paths, "icons")
__buttons__: str = getattr(Paths, "buttons")
