from .__set_data__ import VideoFile

__video_name__: str = getattr(VideoFile, "name")
__video_ext__: str = getattr(VideoFile, "ext")
__video_path__: str = getattr(VideoFile, "path")
__video_directory__: str = getattr(VideoFile, "directory")
